@@ -2,8 +2,11 @@ __all__ = [
     'PredictionSummaryComponent',
     'ImportancesComponent',
     'PdpComponent',
+    'WhatIfComponent',
 ]
 
+import pandas as pd
+
 import dash
 import dash_core_components as dcc
 import dash_bootstrap_components as dbc
@@ -13,7 +16,9 @@ import dash_table
 from dash.dependencies import Input, Output, State
 from dash.exceptions import PreventUpdate
 
+
 from .dashboard_methods import *
+from ..explainer_methods import get_feature_dict
 
 class PredictionSummaryComponent(ExplainerComponent):
     def __init__(self, explainer, title="Prediction Summary", name=None,
@@ -207,7 +212,8 @@ class ImportancesComponent(ExplainerComponent):
             dbc.Row([
                 dbc.Col([
                     dcc.Loading(id='loading-importances-graph-'+self.name, 
-                            children=[dcc.Graph(id='importances-graph-'+self.name)])
+                            children=[dcc.Graph(id='importances-graph-'+self.name,
+                                                config=dict(modeBarButtons=[['toImage']], displaylogo=False))])
                 ]),
             ]), 
         ])
@@ -324,7 +330,8 @@ class PdpComponent(ExplainerComponent):
                 dbc.Row([
                     dbc.Col([
                         dcc.Loading(id='loading-pdp-graph-'+self.name, 
-                            children=[dcc.Graph(id='pdp-graph-'+self.name)]),
+                            children=[dcc.Graph(id='pdp-graph-'+self.name,
+                                                config=dict(modeBarButtons=[['toImage']], displaylogo=False))]),
                     ])
                 ]),
                 dbc.Row([
@@ -388,4 +395,151 @@ class PdpComponent(ExplainerComponent):
             col_options = [{'label': col, 'value':col} 
                                 for col in self.explainer.columns_ranked_by_shap(cats, pos_label=pos_label)]
             return col_options
+
+
+class WhatIfComponent(ExplainerComponent):
+    def __init__(self, explainer, title="What if...", name=None,
+                    hide_title=False, hide_index=False, hide_selector=False,
+                    hide_contributions=False, hide_pdp=False,
+                    index=None, pdp_col=None, pos_label=None):
+        """Interaction Dependence Component.
+
+        Args:
+            explainer (Explainer): explainer object constructed with either
+                        ClassifierExplainer() or RegressionExplainer()
+            title (str, optional): Title of tab or page. Defaults to 
+                        "What if...".
+            name (str, optional): unique name to add to Component elements. 
+                        If None then random uuid is generated to make sure 
+                        it's unique. Defaults to None.
+            hide_title (bool, optional): hide the title
+            hide_index (bool, optional): hide the index selector
+            hide_selector (bool, optional): hide the pos_label selector
+            hide_contributions (bool, optional): hide the contributions graph
+            hide_pdp (bool, optional): hide the pdp graph
+            index (str, int, optional): default index
+            pdp_col (str, optional): default pdp feature col
+            pos_label ({int, str}, optional): initial pos label. 
+                        Defaults to explainer.pos_label
+            
+        """
+        super().__init__(explainer, title, name)
+        
+        self.hide_title, self.hide_index = hide_title, hide_index
+        self.hide_selector = hide_selector
+        self.hide_contributions, self.hide_pdp = hide_contributions, hide_pdp
+        self.index, self.pdp_col = index, pdp_col
+        
+        if self.pdp_col is None:
+            self.pdp_col = self.explainer.columns_ranked_by_shap(cats=True)[0]
+            
+        self.index_name = 'whatif-index-'+self.name
+        
+        self._input_features = self.explainer.columns_cats
+        self._feature_inputs = [
+            self._generate_dash_input(feature, self.explainer.columns, self.explainer.cats) 
+                                       for feature in self._input_features]
+        self._feature_callback_inputs = [Input('whatif-'+feature+'-input-'+self.name, 'value') for feature in self._input_features]
+        self._feature_callback_outputs = [Output('whatif-'+feature+'-input-'+self.name, 'value') for feature in self._input_features]
+        
+        self.selector = PosLabelSelector(explainer, name=self.name, pos_label=pos_label)
+        
+        self.register_dependencies('preds', 'shap_values')
+        
+        
+    def _generate_dash_input(self, col, cols, cats):
+        fd = get_feature_dict(cols, cats)
+        if col in cats:
+            return html.Div([
+                html.P(col),
+                dcc.Dropdown(id='whatif-'+col+'-input-'+self.name, 
+                             options=[dict(label=cat[len(col)+1:], value=cat[len(col)+1:]) for cat in fd[col]],
+                             clearable=False)
+            ])
+        else:
+            return  html.Div([
+                        html.P(col),
+                        dbc.Input(id='whatif-'+col+'-input-'+self.name, type="number"),
+                    ])
+        
+    def layout(self):
+        return dbc.Container([
+            dbc.Row([
+                make_hideable(
+                    dbc.Col([
+                        html.H1("What if...analysis")
+                    ]), hide=self.hide_title)
+            ]),
+            dbc.Row([
+                make_hideable(
+                        dbc.Col([
+                            dbc.Label("Index:"),
+                            dcc.Dropdown(id='whatif-index-'+self.name, 
+                                options = [{'label': str(idx), 'value':idx} 
+                                                for idx in self.explainer.idxs],
+                                value=self.index)
+                        ], md=4), hide=self.hide_index), 
+                make_hideable(
+                        dbc.Col([self.selector.layout()
+                    ], md=2), hide=self.hide_selector),
+                ], form=True),
+            dbc.Row([
+                dbc.Col([
+                    html.H3("Edit Feature input:")
+                ])
+            ]),
+            dbc.Row([
+                dbc.Col([
+                    *self._feature_inputs[:int((len(self._feature_inputs) + 1)/2)]
+                ]),
+                dbc.Col([
+                    *self._feature_inputs[int((len(self._feature_inputs) + 1)/2):]
+                ]),
+            ]),
+            dbc.Row([
+                make_hideable(
+                    dbc.Col([
+                        html.H3("Prediction and contributions:"),
+                        dcc.Loading(id='loading-whatif-contrib-graph-'+self.name, 
+                            children=[dcc.Graph(id='whatif-contrib-graph-'+self.name,
+                                                config=dict(modeBarButtons=[['toImage']], displaylogo=False))]),
+                    ]), hide=self.hide_contributions),
+                make_hideable(
+                    dbc.Col([
+                        html.H3("Partial dependence:"),
+                        dcc.Dropdown(id='whatif-pdp-col-'+self.name, 
+                                     options=[dict(label=col, value=col) for col in self._input_features],
+                                     value=self.pdp_col),
+                        dcc.Loading(id='loading-whatif-pdp-graph-'+self.name, 
+                            children=[dcc.Graph(id='whatif-pdp-graph-'+self.name, 
+                                    config=dict(modeBarButtons=[['toImage']], displaylogo=False))]),
+                    ]), hide=self.hide_pdp),
+            ])
+        ], fluid=True)
+        
+    def _register_callbacks(self, app):
+        @app.callback(
+            [Output('whatif-contrib-graph-'+self.name, 'figure'),
+             Output('whatif-pdp-graph-'+self.name, 'figure')],
+            [Input('whatif-pdp-col-'+self.name, 'value'),
+             Input('pos-label-'+self.name, 'value'),
+             *self._feature_callback_inputs,
+             ],
+        )
+        def update_whatif_plots(pdp_col, pos_label, *input_args):
+            X_row = pd.DataFrame(dict(zip(self._input_features, input_args)), index=[0]).fillna(0)
+            contrib_plot = self.explainer.plot_shap_contributions(X_row=X_row, pos_label=pos_label)
+            pdp_plot = self.explainer.plot_pdp(pdp_col, X_row=X_row, pos_label=pos_label)
+            return contrib_plot, pdp_plot
+        
+        @app.callback(
+            [*self._feature_callback_outputs],
+            [Input('whatif-index-'+self.name, 'value')]
+        )
+        def update_whatif_inputs(index):
+            idx = self.explainer.get_int_idx(index)
+            if idx is None:
+                raise PreventUpdate
+            feature_values = self.explainer.X_cats.iloc[[idx]].values[0].tolist()
+            return feature_values
                         
