@@ -1,7 +1,8 @@
 __all__ = ['titanic_survive', 
             'titanic_fare', 
             'titanic_embarked', 
-            'titanic_names']
+            'titanic_names',
+            'feature_descriptions']
 
 import numpy as np
 import pandas as pd
@@ -10,19 +11,34 @@ from pathlib import Path
 d_train = pd.read_csv(Path(__file__).resolve().parent / 'datasets'/ 'titanic_train.csv')
 d_test = pd.read_csv(Path(__file__).resolve().parent / 'datasets'/'titanic_test.csv')
 
+feature_descriptions = {
+    "Sex": "Gender of passenger",
+    "Deck": "The deck the passenger had their cabin on",
+    "PassengerClass": "The class of the ticket: 1st, 2nd or 3rd class",
+    "Fare": "The amount of money people paid", 
+    "No_of_relatives_on_board": "number of siblings, spouses, parents plus children on board",
+    "Embarked": "the port where the passenger boarded the Titanic. Either Southampton, Cherbourg or Queenstown",
+    "Age": "Age of the passenger",
+    "No_of_siblings_plus_spouses_on_board": "The sum of the number of siblings plus the number of spouses on board",
+    "No_of_parents_plus_children_on_board" : "The sum of the number of parents plus the number of children on board",
+}
 
 def titanic_survive():
-    X_train = d_train.drop(['Survived', 'Name'], axis=1)
-    y_train = d_train['Survived']
-    X_test = d_test.drop(['Survived', 'Name'], axis=1)
-    y_test = d_test['Survived']
+    X_train = d_train.drop(['Survival', 'Name'], axis=1)
+    X_train.index = d_train.Name
+    y_train = d_train['Survival']
+    X_test = d_test.drop(['Survival', 'Name'], axis=1)
+    X_test.index = d_test.Name
+    y_test = d_test['Survival']
     return X_train, y_train, X_test, y_test
 
 
 def titanic_fare():
     X_train = d_train.drop(['Fare', 'Name'], axis=1)
+    X_train.index = d_train.Name
     y_train = d_train['Fare']
     X_test = d_test.drop(['Fare', 'Name'], axis=1)
+    X_test.index = d_test.Name
     y_test = d_test['Fare']
     return X_train, y_train, X_test, y_test
 
@@ -32,6 +48,7 @@ def titanic_embarked():
     d_train2 = d_train2[d_train2.Embarked_Unknown==0]
     X_train = d_train2.drop(['Embarked_Cherbourg', 'Embarked_Queenstown', 
         'Embarked_Southampton', 'Embarked_Unknown', 'Name'], axis=1)
+    X_train.index = d_train2.Name
     y_train = pd.Series(np.where(d_train2.Embarked_Queenstown==1, 0, 
                             np.where(d_train2.Embarked_Southampton==1, 1, 
                                 np.where(d_train2.Embarked_Cherbourg==1, 2, 3))), 
@@ -39,6 +56,7 @@ def titanic_embarked():
                     name="Embarked")
     X_test = d_test.drop(['Embarked_Cherbourg', 'Embarked_Queenstown', 
         'Embarked_Southampton', 'Embarked_Unknown', 'Name'], axis=1)
+    X_test.index = d_test.Name
     y_test = pd.Series(np.where(d_test.Embarked_Queenstown==1, 0, 
                             np.where(d_test.Embarked_Southampton==1, 1, 
                                 np.where(d_test.Embarked_Cherbourg==1, 2, 3))), 
@@ -47,5 +65,9 @@ def titanic_embarked():
     return X_train, y_train, X_test, y_test
 
 
-def titanic_names():
+def titanic_names(train_only=False, test_only=False):
+    if train_only:
+        return d_train['Name'].values.tolist()
+    if test_only:
+        d_test['Name'].values.tolist()
     return (d_train['Name'].values.tolist(), d_test['Name'].values.tolist())
